/************************************************************/
/**                                                        **/
/**   NAME       : scotch.h                                **/
/**                                                        **/
/**   AUTHOR     : Francois PELLEGRINI                     **/
/**                                                        **/
/**   FUNCTION   : Declaration file for the LibScotch      **/
/**                static mapping and sparse matrix block  **/
/**                ordering sequential library.            **/
/**                                                        **/
/**   DATES      : # Version 3.2  : from : 07 sep 1996     **/
/**                                 to     22 aug 1998     **/
/**                # Version 3.3  : from : 02 oct 1998     **/
/**                                 to     31 may 1999     **/
/**                # Version 3.4  : from : 10 oct 1999     **/
/**                                 to     15 nov 2001     **/
/**                                                        **/
/************************************************************/

/*
**  The type and structure definitions.
*/

/*+ Integer type. +*/

typedef int SCOTCH_Num;

#define SCOTCH_NUMMAX               MAXINT

/*+ Opaque objects. The dummy sizes of these objects,
computed at compile-time by program "dummysizes",
are given as double values for proper padding +*/

typedef struct {
  double                    dummy[1];
} SCOTCH_Strat;

typedef struct {
  double                    dummy[4];
} SCOTCH_Arch;

typedef struct {
  double                    dummy[8];
} SCOTCH_Geom;

typedef struct {
  double                    dummy[7];
} SCOTCH_Graph;

typedef struct {
  double                    dummy[17];
} SCOTCH_Mapping;

typedef struct {
  double                    dummy[5];
} SCOTCH_Ordering;

/*
**  The function prototypes.
*/

void                        SCOTCH_errorProg    (const char * const);
void                        SCOTCH_errorPrint   (const char * const, ...);
void                        SCOTCH_errorPrintW  (const char * const, ...);

int                         SCOTCH_stratInit    (SCOTCH_Strat * const);
void                        SCOTCH_stratExit    (SCOTCH_Strat * const);
int                         SCOTCH_stratSave    (const SCOTCH_Strat * const, FILE * const);
int                         SCOTCH_stratBipart  (SCOTCH_Strat * const, const char * const);
int                         SCOTCH_stratMap     (SCOTCH_Strat * const, const char * const);
int                         SCOTCH_stratOrder   (SCOTCH_Strat * const, const char * const);

int                         SCOTCH_archInit     (SCOTCH_Arch * const);
void                        SCOTCH_archExit     (SCOTCH_Arch * const);
int                         SCOTCH_archLoad     (SCOTCH_Arch * const, FILE * const);
int                         SCOTCH_archSave     (const SCOTCH_Arch * const, FILE * const);
int                         SCOTCH_archBuild    (SCOTCH_Arch * const, const SCOTCH_Graph * const, const SCOTCH_Num, const SCOTCH_Num * const, const SCOTCH_Strat * const);
int                         SCOTCH_archCmplt    (SCOTCH_Arch * const, const SCOTCH_Num);
char *                      SCOTCH_archName     (const SCOTCH_Arch * const);

int                         SCOTCH_geomInit     (SCOTCH_Geom * const);
void                        SCOTCH_geomExit     (SCOTCH_Geom * const);
int                         SCOTCH_geomLoadChac (SCOTCH_Geom * const, FILE * const, FILE * const, const char * const);
int                         SCOTCH_geomSaveChac (const SCOTCH_Geom * const, FILE * const, FILE * const, const char * const);
int                         SCOTCH_geomLoadHabo (SCOTCH_Geom * const, FILE * const, FILE * const, const char * const);
int                         SCOTCH_geomLoadScot (SCOTCH_Geom * const, FILE * const, FILE * const, const char * const);
int                         SCOTCH_geomSaveScot (const SCOTCH_Geom * const, FILE * const, FILE * const, const char * const);

int                         SCOTCH_graphInit    (SCOTCH_Graph * const);
void                        SCOTCH_graphExit    (SCOTCH_Graph * const);
int                         SCOTCH_graphLoad    (SCOTCH_Graph * const, FILE * const, const SCOTCH_Num, const SCOTCH_Num);
int                         SCOTCH_graphSave    (const SCOTCH_Graph * const, FILE * const);
int                         SCOTCH_graphBuild   (SCOTCH_Graph * const, const SCOTCH_Num, const SCOTCH_Num * const, const SCOTCH_Num * const, const SCOTCH_Num * const, const SCOTCH_Num, const SCOTCH_Num * const, const SCOTCH_Num * const, const SCOTCH_Num, const SCOTCH_Num);
int                         SCOTCH_graphCheck   (const SCOTCH_Graph * const);
void                        SCOTCH_graphSize    (const SCOTCH_Graph * const, SCOTCH_Num * const, SCOTCH_Num * const);
void                        SCOTCH_graphData    (const SCOTCH_Graph * const, SCOTCH_Num * const, SCOTCH_Num ** const, SCOTCH_Num ** const, SCOTCH_Num ** const, SCOTCH_Num * const, SCOTCH_Num ** const, SCOTCH_Num ** const, SCOTCH_Num * const, SCOTCH_Num * const);
void                        SCOTCH_graphStat    (const SCOTCH_Graph * const, SCOTCH_Num * const, SCOTCH_Num * const, SCOTCH_Num * const, double * const, double * const, SCOTCH_Num * const, SCOTCH_Num * const, double * const, double * const, SCOTCH_Num * const, SCOTCH_Num * const, SCOTCH_Num * const, double * const, double * const);

int                         SCOTCH_graphPart    (const SCOTCH_Graph * const, const SCOTCH_Strat * const, const SCOTCH_Num, SCOTCH_Num * const);
int                         SCOTCH_graphMap     (const SCOTCH_Graph * const, const SCOTCH_Strat * const, const SCOTCH_Arch * const, SCOTCH_Num * const);

int                         SCOTCH_graphOrder   (const SCOTCH_Graph * const, const SCOTCH_Strat * const, SCOTCH_Num * const, SCOTCH_Num * const, SCOTCH_Num * const, SCOTCH_Num * const);
int                         SCOTCH_graphOrderList (const SCOTCH_Graph * const, const SCOTCH_Num, const SCOTCH_Num * const, const SCOTCH_Strat * const, SCOTCH_Num * const, SCOTCH_Num * const, SCOTCH_Num * const, SCOTCH_Num * const);

int                         SCOTCH_mapInit      (SCOTCH_Mapping * const, const SCOTCH_Graph * const, const SCOTCH_Arch * const, SCOTCH_Num * const);
void                        SCOTCH_mapExit      (SCOTCH_Mapping * const);
int                         SCOTCH_mapSave      (const SCOTCH_Mapping * const, FILE * const);
int                         SCOTCH_mapView      (const SCOTCH_Mapping * const, FILE * const);
int                         SCOTCH_mapCompute   (SCOTCH_Mapping * const, const SCOTCH_Strat * const);

int                         SCOTCH_orderInit    (SCOTCH_Ordering * const, const SCOTCH_Graph * const, SCOTCH_Num * const, SCOTCH_Num * const, SCOTCH_Num * const, SCOTCH_Num * const);
void                        SCOTCH_orderExit    (SCOTCH_Ordering * const);
int                         SCOTCH_orderSave    (const SCOTCH_Ordering * const, FILE * const);
int                         SCOTCH_orderSaveM   (const SCOTCH_Ordering * const, FILE * const);
int                         SCOTCH_orderCheck   (const SCOTCH_Ordering * const);
int                         SCOTCH_orderFactor  (const SCOTCH_Ordering * const, SCOTCH_Graph * const);
int                         SCOTCH_orderView    (const SCOTCH_Ordering * const, FILE * const);
int                         SCOTCH_orderCompute (SCOTCH_Ordering * const, const SCOTCH_Strat * const);
int                         SCOTCH_orderComputeList (SCOTCH_Ordering * const, const SCOTCH_Num, const SCOTCH_Num * const, const SCOTCH_Strat * const);
